#include <stdio.h>
#include <unistd.h>
#include <sys/file.h>
#include <xtables.h>
#include "xshared.h"

#define XT_LOCK_NAME "/var/run/xtables"

/*
 * Print out any special helps. A user might like to be able to add a --help
 * to the commandline, and see expected results. So we call help for all
 * specified matches and targets.
 */
void print_extension_helps(const struct xtables_target *t,
    const struct xtables_rule_match *m)
{
	for (; t != NULL; t = t->next) {
		if (t->used) {
			printf("\n");
			if (t->help == NULL)
				printf("%s does not take any options\n",
				       t->name);
			else
				t->help();
		}
	}
	for (; m != NULL; m = m->next) {
		printf("\n");
		if (m->match->help == NULL)
			printf("%s does not take any options\n",
			       m->match->name);
		else
			m->match->help();
	}
}

bool xtables_lock(bool wait)
{
	int i = 0, ret, fd; 

	fd = open(XT_LOCK_NAME, O_CREAT, 0600);
	if (fd < 0)
		return true;

	while (1) {
		ret = flock(fd, LOCK_EX | LOCK_NB);
		if (ret == 0)
			return true;
		else if (wait == false)
			return false;
		if (++i % 10 == 0)
			fprintf(stderr, "Another app is currently holding the xtables lock; "
				"waiting for it to exit...\n");
		sleep(1);
	}
}
